"""Application configuration file for django-helcim."""
from django.apps import AppConfig


class HelcimConfig(AppConfig):
    """Configuration details for django-helcim."""
    name = 'helcim'
    verbose_name = 'django-helcim'
